var map; //Remembers the map
var Coordinates, ZoomLevel=$I(GetCookie('MapZoom', 16)); //Settings
var UsersInfo={}; //List of markers IDs with the information UserID:{Marker:GMarker, AccessKeyIndex:, AccessKey:, UserID:, UserName:, AccessKey:, Latitude:, Longitude:, Altitude:, Accuracy:, RealUpdateTime:}
var AccessKeys=[]; //The number of displayed access keys
var LastUpdate=0, AddedNewUser=false; //The last time an update occurred (and if it needs to be reset to 0)
var UserInfoFields={UserID:1,UserName:1,AccessKey:1}; //This must be in the order of the received information from "GetUsersInfo"
var UserDisplayFields={UserID:'ID', AccessKey:'Access Key', UserName:'User Name', Latitude:'Latitude', Longitude:'Longitude', Altitude:'Altitude', Accuracy:'Accuracy', RealUpdateTime:'Update Time'}; //Name of user fields to display in popup

//Initialize the map
function initialize(DefaultMapCenter)
{
	Coordinates=GetCookie('MapCenter', DefaultMapCenter).split(/,/);

	//Initialize the Google Map
	if(GBrowserIsCompatible())
	{
		map=new GMap2($('map_canvas'));
		map.setCenter(new GLatLng(Coordinates[0], Coordinates[1]), ZoomLevel);
		map.setUIToDefault();
		GetUsersInfo(GetCookie('AccessKeys', ''), false);
	}
}

//Update the displayed markers
var TheActionCounter;
function ActionCounter_Execute()
{
	//If a new user was added, refresh all users
	if(AddedNewUser)
	{
		AddedNewUser=false;
		LastUpdate=0;
	}

	//Execute the action
	if(AccessKeys.length)
		GetURL('Query.php', {Action:'GetUpdates', LastUpdate:LastUpdate, AccessKeys:AccessKeys.join(',')}, {CallBack:MarkersUpdated, UseCache:false});
	else
		TheActionCounter.ActionFinished();
}
function MarkersUpdated(Data)
{
	//Make sure return was successful
	if(!TestReturnData(Data))
		return TheActionCounter.ActionFinished();

	//Parse the data
	Data=Data.substr(1).split(';');
	LastUpdate=Data[0];
	var NumUpdated=Data[1];
	Data.splice(0,2);

	if(NumUpdated==0) //If no updates...
		UpdateLog('No updates');

	//Update the markers
	var NewInfoVars=['UserID', 'Latitude', 'Longitude', 'Altitude', 'Accuracy', 'RealUpdateTime'];
	for(var i=0;i<NumUpdated;i++)
	{
		//Parse the received user data
		var NewInfo={};
		for(var n=0;n<NewInfoVars.length;n++)
			NewInfo[NewInfoVars[n]]=Data[i*NewInfoVars.length+n];

		//Make sure the user information exists and update it
		var UserInfo=UsersInfo[NewInfo.UserID];
		if(UserInfo===undefined)
			UserInfo=UsersInfo[NewInfo.UserID]={UserID:NewInfo.UserID,UserName:'Please resync with database'};
		$CM(UserInfo, NewInfo);

		//Update the marker
		if(UserInfo.hasOwnProperty('Marker')) //Just update a marker if it already exists
		{
			UserInfo.Marker.setLatLng(new GLatLng(UserInfo.Latitude, UserInfo.Longitude));
			UpdateLog('Updated User Position: '+UserInfo.UserName);
		}
		else //Create a new marker
		{
			var MarkerIcon=$CM(new GIcon(), {
				image:'GetIcon.php?Num='+(UserInfo.AccessKeyIndex+1),
				shadow:'Res/Shadow.png',
				iconSize:new GSize(30,44),
				shadowSize:new GSize(46,44),
				iconAnchor:new GPoint(16,43),
				infoWindowAnchor:new GPoint(31, 0)
			});
			var MyMarker=new GMarker(new GLatLng(UserInfo.Latitude, UserInfo.Longitude), {title:UserInfo.UserName, icon:MarkerIcon});
			MyMarker.UserID=UserInfo.UserID;GEvent.addListener(MyMarker, 'click', function() { ShowUserData(this); });
			map.addOverlay(MyMarker);
			UserInfo.Marker=MyMarker;
			UpdateLog('Created User Position: '+UserInfo.UserName);
		}
	}

	TheActionCounter.ActionFinished();
}

//Make sure AJAX return was successful
function TestReturnData(Data)
{
	if(Data.substr(0,1)!='S') //On error
		if(Data=='')
			return false;
		else
		{
			var IsError=(Data.substr(0,1)=='E');
			alert('Error from server'+(IsError ? ': ' : ' (Unformatted error): ')+Data.substr(IsError ? 1 : 0));
			return false;
		}
	return true;
}

//Update the action log
var LogBuffer=[], LogBufferLength=10;
function UpdateLog(Str)
{
	if(LogBuffer.length>=LogBufferLength)
		LogBuffer.pop();
	LogBuffer.unshift(GetTimeString(new Date())+': '+Str);
	//$('Updates').innerHTML=LogBuffer.join('<br>'); //I removed this for now as there is no reason (or mechanism) to unhide the updates log on the interface
}

//Get all the users' info
function GetUsersInfo(TheAccessKeys, AddingNewUser)
{
	if(TheAccessKeys=='')
		return;

	GetURL('Query.php', {Action:'GetUsersInfo', AccessKeys:TheAccessKeys}, {CallBack:UsersInfoUpdated, UseCache:false, ExtraData:AddingNewUser});
}
function UsersInfoUpdated(Data, AddingNewUser)
{
	//Make sure return was successful
	if(!TestReturnData(Data))
		return;

	//Check for no returned information
	if(Data.length<2)
	{
		if(AddingNewUser)
			alert('Unknown Access Key');
		return;
	}

	//Process each user's data
	Data=Data.substr(1).split(',');
	for(var i=0;i<Data.length;i++)
	{
		//Parse the received user data
		var IncomingUserInfo=Data[i].split(';'), UserID=IncomingUserInfo[0], AccessKey=IncomingUserInfo[2];

		//Make sure the user information exists (skip this user if they already exist)
		var UserInfo=UsersInfo[UserID];
		if(UserInfo!==undefined)
			continue;
		UserInfo=UsersInfo[UserID]={};

		//Add the access key to the access keys list
		AccessKeys.push(AccessKey);
		UserInfo.AccessKeyIndex=AccessKeys.length-1;

		//Update the user information
		var n=0;
		for(var FieldName in UserInfoFields)
			UserInfo[FieldName]=IncomingUserInfo[n++];
	}
	SetCookie('AccessKeys', AccessKeys.join(',')); //Update the access keys in the cookies

	//Continue loading the rest of the interface
	CreateUserLegend();
	UpdateLog(AddingNewUser ? 'Added new user' : 'All users’ information retrieved');
	if(TheActionCounter===undefined)
		TheActionCounter=new ActionCounter('IntervalValue', 'LastUpdate', 'TimeLeft', ActionCounter_Execute);

	//Force a new user to be loaded
	if(AddingNewUser)
	{
		AddedNewUser=true;
		TheActionCounter.ExecuteAction();
	}
}

//Create the user legend
function CreateUserLegend()
{
	Data=[];
	for(var UserID in UsersInfo)
	{
		var Index=UsersInfo[UserID].AccessKeyIndex;
		Data[Index]=(Index+1)+')</td><td><a href="#" onclick="return DisplayUser('+UserID+')">'+UsersInfo[UserID].UserName+'</a>';
	}
	$('UserLegend').innerHTML='<table><tr><td>'+Data.join('</td></tr><tr><td>')+'</td></tr></table>';
}

//Show the user's information in a popup
function ShowUserData(TheMarker)
{
	//Process the user's data
	var UserInfo=UsersInfo[TheMarker.UserID];
	var Data=[];
	for(var FieldName in UserDisplayFields)
	{
		var TheData=UserInfo[FieldName];
		switch(FieldName)
		{
			case 'Accuracy':
				TheData+=" meters"; break;
			case 'RealUpdateTime':
				TheData=GetTimeString($I(TheData)*1000).replace(/ (\d+):/, '<br>$1:'); break;
		}

		Data.push(UserDisplayFields[FieldName]+'</th><td>'+TheData);
	}

	//Show the user's data
	TheMarker.openInfoWindowHtml('<table class=UserInfoTable><tr><th>'+Data.join('</td></tr><tr><th>')+'</td></tr></table>');
}

//Map Coordinates
var IsSearchBarVisible=false;
function StoreMapCenter() //Store center of map
{
	var NewCenter=map.getCenter();
	SetCookie('MapCenter', NewCenter.lat()+','+NewCenter.lng());
	SetCookie('MapZoom', map.getZoom());
	UpdateLog('Map center stored');
}
function GetCookie(Name, Default)
{
	var Matches=document.cookie.match(new RegExp('(?:;|^)\\s*'+Name+'=(.*?)(?:;|$)'));
	return (Matches ? unescape(Matches[1]) : Default);
}
function SetCookie(Name, Value)
{
	document.cookie=Name+'='+escape(Value)+'; expires='+(new Date(new Date().getTime()+(365*24*60*60*1000))).toGMTString()+'; path=/';
}
function DisplaySearchBar()
{
	IsSearchBarVisible=!IsSearchBarVisible;
	$('DisplaySearchBar').value=(IsSearchBarVisible ? 'Hide' : 'Show')+' Search Bar';
	map[IsSearchBarVisible ? 'enableGoogleBar' : 'disableGoogleBar']();
}

//Search for a user on the map
function DisplayUser(UserID)
{
	try
	{
		var UserInfo=UsersInfo[UserID];
		if(UserInfo===undefined || UserInfo.Marker===undefined)
			alert('Invalid User ID (Or user position not yet loaded)');
		else
			ShowUserData(UserInfo.Marker);
	} catch(e) {}
	return false;
}

//Add a user to the map
function AddUser()
{
	var AccessKey=$('AddUserText').value.replace(/^\s+|\s+$/g, ''); //Get trimmed access key
	if(!AccessKey.match(/^[A-Za-z0-9]{6}$/))
		alert('Invalid Access Key');
	else
		GetUsersInfo(AccessKey, true);
}